# from xprompt.client_services.dummy_parser import DummyParser
from xprompt.utils import send_request_with_file
from xprompt_common.data_utils import FilePathType, identify_path_type
from xprompt_common.base_service import BaseService
from xprompt_common.prompt_parser import PromptParser

TAG_TO_SERVICE = {
    # "dummy": DummyParser,
}

try:
    pass

    from xprompt.client_services.llamahub_connectors.google_docs import GoogleDoc

    TAG_TO_SERVICE.update(
        {
            "gdoc": GoogleDoc,
        }
    )
except ImportError:
    pass


class ClientPromptParser(PromptParser):
    tag_to_service: dict[str, BaseService.__class__] = TAG_TO_SERVICE

    def handle_src_in_tag(self):
        """
        Upload file to server and create mapping between s3 path and original path
        Args:
            src:

        Returns:
            new s3 path
        """
        local_file_paths = []
        for tag in self.tags:
            if (
                "src" in tag.attrs
                and identify_path_type(tag["src"]) == FilePathType.LOCAL
            ):
                local_file_paths.append(tag["src"])

        if len(local_file_paths) > 0:
            response = send_request_with_file(
                file_paths=local_file_paths, endpoint="upload_files/"
            )
            s3_paths = response["s3_paths"]
            local_file_to_s3_path = {
                local_file_path: s3_path
                for local_file_path, s3_path in zip(local_file_paths, s3_paths)
            }

            for tag in self.tags:
                if (
                    "src" in tag.attrs
                    and identify_path_type(tag["src"]) == FilePathType.LOCAL
                ):
                    if tag["src"] in local_file_to_s3_path:
                        tag["src"] = local_file_to_s3_path[tag["src"]]
                    else:
                        raise RuntimeError(f"Failed to handle {tag['src']}")



import xprompt
from xprompt.utils import send_request_with_json


class OutputConvertorClient:
    @classmethod
    def convert(cls, text, output_type, **kwargs):
        timeout = kwargs.pop("timeout", None)
        payload = {"text": text, "output_format": output_type, **kwargs}

        response = send_request_with_json(
            json_payload=payload, endpoint="convert_output", timeout=timeout
        )
        return response.content


if __name__ == "__main__":
    res = OutputConvertorClient()
    xprompt.api_key = ""
    content = res.convert("sure, this is something", "audio")
    with open("./test.mp3", "wb") as f:
        f.write(content)

    print("done")



import xprompt
from xprompt.client_prompt_parser import ClientPromptParser
from xprompt.data_client import OutputConvertorClient
from xprompt.schemas import XPromptOutput
from xprompt.utils import send_request_with_json


class OpenAIGenerate:
    output_convertor: OutputConvertorClient = OutputConvertorClient()

    @staticmethod
    def client_prompt_parsing(messages):
        for message in messages:
            if message["role"] != "assistant":
                prompt = message["content"]
                prompt_parser = ClientPromptParser(prompt=prompt)

                # upload file to s3
                prompt_parser.handle_src_in_tag()

                # run local parsing
                message["content"], parsing_info = prompt_parser.run_services()

    @classmethod
    def create(cls, **kwargs):
        timeout = kwargs.pop("timeout", None)
        kwargs["client_type"] = "chat_completion"
        kwargs["openai_api_key"] = xprompt.openai_api_key
        output_type = kwargs.pop("output_format", None)

        # client side parsing
        cls.client_prompt_parsing(kwargs["messages"])

        # send xprompt api request
        response = send_request_with_json(
            json_payload=kwargs, endpoint="generate/", timeout=timeout
        )

        generation_response = response.json()
        if output_type and generation_response["choices"]:
            for message in generation_response["choices"]:
                content = message["message"].get("content")
                if not content:
                    continue

                byte_content = cls.output_convertor.convert(
                    text=message["message"]["content"], output_type=output_type
                )
                message["message"]["byte_content"] = byte_content

        return XPromptOutput(generation_response)


class ChatCompletion(OpenAIGenerate):
    """
    Replace of openai.ChatCompletion
    """

    @classmethod
    def create(cls, **kwargs):
        kwargs["client_type"] = "chat_completion"
        return super().create(**kwargs)


class Completion(OpenAIGenerate):
    """
    Replace of openai.Completion
    """

    @classmethod
    def create(cls, **kwargs):
        kwargs["client_type"] = "completion"
        return super().create(**kwargs)


class Embedding(OpenAIGenerate):
    """
    Replace of openai.Embedding
    """

    @classmethod
    def create(cls, **kwargs):
        kwargs["client_type"] = "embedding"
        return super().create(**kwargs)



"""
Create google oauth credentials
https://developers.google.com/docs/api/quickstart/python

1. Enable API
2. Create OAuth 2.0 Client using web application
3. download and rename to credentials.json, and then place is next to where you run xprompt
"""
from typing import Optional

from xprompt_common.api_schema.generate_schema import ParsingInfo
from xprompt_common.base_service import BaseService
from llama_index import download_loader


class GoogleDoc(BaseService):
    doc_id: str = ""

    def run(self) -> (str, Optional[ParsingInfo]):
        google_docs_reader = download_loader("GoogleDocsReader")

        gdoc_ids = [self.doc_id]
        loader = google_docs_reader()
        documents = loader.load_data(document_ids=gdoc_ids)
        if len(documents) == 0:
            return ""

        return documents[0].text, None



import time
from typing import Any

import requests

import xprompt
from xprompt.constants import get_backend_endpoint
from xprompt_common.errors import TryAgain


def exclude_json_none(json_dict: dict[str, Any]) -> dict[str, Any]:
    """
    Delete keys with the value ``None`` in a dictionary, recursively.

    This alters the input so you may wish to ``copy`` the dict first.
    """
    # For Python 3, write `list(d.items())`; `d.items()` won’t work
    # For Python 2, write `d.items()`; `d.iteritems()` won’t work
    for key, value in list(json_dict.items()):
        if value is None:
            del json_dict[key]
        elif isinstance(value, dict):
            exclude_json_none(value)
    return json_dict


def send_request_with_json(
    json_payload: dict[str, Any], endpoint: str, timeout: int = 10
):
    start = time.time()
    backend_endpoint = get_backend_endpoint(xprompt.debug)
    while True:
        try:
            headers = {
                "Content-Type": "application/json; charset=utf-8",
                "Authorization": f"Bearer {xprompt.api_key}",
            }

            response = requests.post(
                f"{backend_endpoint}/{endpoint}", headers=headers, json=json_payload
            )
            if response.status_code == 409:
                raise TryAgain("failed with conflict")

            if response.status_code >= 300:
                raise RuntimeError(response.text)

            return response
        except TryAgain:
            if timeout is not None and time.time() > start + timeout:
                raise
            time.sleep(1)


def send_request_with_file(file_paths: list[str], endpoint: str, timeout: int = 10):
    start = time.time()
    backend_endpoint = get_backend_endpoint(xprompt.debug)
    while True:
        try:
            headers = {
                "Authorization": f"Bearer {xprompt.api_key}",
            }

            files = [("files", open(fp, "rb")) for fp in file_paths]
            response = requests.post(
                f"{backend_endpoint}/{endpoint}", headers=headers, files=files
            )
            if response.status_code == 409:
                raise TryAgain("failed with conflict")

            if response.status_code >= 300:
                raise RuntimeError(response.text)

            break
        except TryAgain:
            if timeout is not None and time.time() > start + timeout:
                raise
            time.sleep(1)

    return response.json()



import requests

import xprompt
from xprompt.constants import get_backend_endpoint


def login(user_email: str, password: str):
    backend_endpoint = get_backend_endpoint(xprompt.debug)
    try:
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "accept": "application/json",
        }
        response = requests.post(
            f"{backend_endpoint}/token",
            headers=headers,
            data={"username": user_email, "password": password},
        )

        return response.json()

    except Exception:
        raise



#

BACKEND_ENDPOINT = {
    "dev": "http://127.0.0.1:8000",
    "prod": "http://xprompt-server-env.eba-yvum4nem.us-west-2.elasticbeanstalk.com",
}


def get_backend_endpoint(debug=False):
    """
    Get backend endpoint based on debug mode
    Args:
        debug: Local host if True

    Returns:
    backend endpoint
    """
    return BACKEND_ENDPOINT["dev" if debug else "prod"]



__app_name__ = "xprompt"
__version__ = "0.0.1"

import os
from typing import Optional

from xprompt.admin_client import login
from xprompt.openai_client import (
    ChatCompletion,
    Completion,
    Embedding,
)

api_key = os.environ.get("API_KEY")
openai_api_key = os.environ.get("OPENAI_API_KEY")

# Path of a file with an API key, whose contents can change. Supercedes
# `api_key` if set.  The main use case is volume-mounted Kubernetes secrets,
# which are updated automatically.
api_key_path: Optional[str] = os.environ.get("API_KEY_PATH")

debug = False
log = None  # Set to either 'debug' or 'info', controls console logging


__all__ = [
    "ChatCompletion",
    "Completion",
    "Embedding",
    "login",
    "api_key",
    "openai_api_key",
    "api_key_path",
    "debug",
    "log",
]



from typing import Optional

from xprompt_common.api_schema.generate_schema import ParsingInfo
from xprompt_common.base_service import BaseService


class DummyParser(BaseService):
    val: str = ""

    def run(self) -> (str, Optional[ParsingInfo]):
        return self.val, None
